#include                <ESP8266WiFi.h>

#include                <DallasTemperature.h>
#include                <DNSServer.h>
#include                <ESP8266WebServer.h>
#include                <OneWire.h>
#include                <WiFiManager.h>                     //https://github.com/tzapu/WiFiManager

const int               ONE_WIRE_BUS    = 4;                // Data wire is plugged TO GPIO 4

int                     numberOfDevices;                    // Number of temperature devices found
OneWire                 oneWire(ONE_WIRE_BUS);              // create oneWire object
DallasTemperature       sensors(&oneWire);                  // Pass our oneWire reference to Dallas Temperature.
WiFiServer              server(80);                         // Set web server port number to 80
DeviceAddress           tempDeviceAddress;                  // We'll use this variable to store a found device address

// ----------------------------------------------------------------------------
// function to return deviceAddr as string

char * addr2s(
    DeviceAddress       deviceAddress,
    char *              s,
    int                 maxLen
    ) {

const int               addrLen         = 16;               // size of addr to be returned

int                     i;

strcpy(s, "");

if (maxLen < addrLen + 1)                                   // verify there is enough room for output
    return s;

for (i = 0; i < addrLen/2; i = i + 1) {
    if (deviceAddress[i] < 16)
        strcat(s, "0");
    itoa(deviceAddress[i], s+strlen(s), 16);
    } // for
    strupr(s);

return s;

} // addr2s

// ----------------------------------------------------------------------------
// function to print a device address

void printAddress(
    DeviceAddress       deviceAddress
    ) {

char                    s[20];

Serial.print(addr2s(deviceAddress, s, sizeof(s)));

} // printAddress

// ----------------------------------------------------------------------------
// SETUP

void setup() {

int                     i;
WiFiManager             wifiManager;

Serial.begin(9600);
Serial.println();


//wifiManager.resetSettings();                              //reset saved settings

wifiManager.autoConnect("tempSensorAP");                    //fetch ssid and pass from eeprom and try to connect

Serial.println("WiFi connection to LAN complete");          //if you get here you have connected to the WiFi

// Print local IP address and start web server
Serial.println("");
Serial.println("WiFi connected.");
Serial.println("IP address: ");
Serial.println(WiFi.localIP());

server.begin();

sensors.begin();                                            // Start up the library
numberOfDevices = sensors.getDeviceCount();                 // Grab a count of devices on the wire

// locate devices on the bus
Serial.print("Locating devices...");
Serial.print("Found ");
Serial.print(numberOfDevices, DEC);
Serial.println(" devices.");

// Loop through each device, print out address
for(i = 0; i < numberOfDevices; i = i + 1){
    // Search the wire for address
    if(sensors.getAddress(tempDeviceAddress, i)){
        Serial.print("Found device ");
        Serial.print(i, DEC);
        Serial.print(" with address: ");
        printAddress(tempDeviceAddress);
        Serial.println();
        }
    else {
        Serial.print("Found ghost device at ");
        Serial.print(i, DEC);
        Serial.print(" but could not detect address. Check power and cabling");
        }
    } // for

}

// ----------------------------------------------------------------------------
// LOOP

/*
Here is clean html I'll be using:

<!DOCTYPE html><html>
<meta name="viewport" content="width=device-width, initial-scale=1">
<head>
<body>
<table border=0>
  <tr>
    <td colspan=2 align=center><font size=6>ESP8266<br>Temperature Sensor<br>Monitor</font></td>
  </tr>
  <tr>
  <td colspan=2><hr></td>
  </tr>
  <tr>
    <th align=center><font size=5>Sensor</font></th>
    <th align=center><font size=5>Temperature</font></th>
  </tr>
  <tr>
    <td align=center>Unit 1</td>
    <td align=center>80</td>
  </tr>
  <tr>
    <td align=center>Unit 2</td>
    <td align=center>90</td>
  </tr>
</table>
</body>
</html>
*/

void loop() {

const long              timeoutTime     = 2000;              // timeout time in milliseconds

char                    c;
WiFiClient              client          = server.available();// Listen for incoming clients
unsigned long           currentTime     = millis();         // Current time
String                  header;                             // Variable to store the HTTP request
int                     i;
unsigned long           previousTime    = 0;                // Previous time
char                    s[20];
float                   tempC;

sensors.requestTemperatures();                              // Send the command to get temperatures

if (client) {                                               // If a new client connects,
    Serial.println("New Client.");                          // print a message out in the serial port
    String currentLine = "";                                // make a String to hold incoming data from the client
    currentTime = millis();
    previousTime = currentTime;
    while (client.connected() && currentTime - previousTime <= timeoutTime) { // loop while the client's connected
        currentTime = millis();
        if (client.available()) {                           // if there's bytes to read from the client,
            c = client.read();                              // read a byte, then
            Serial.write(c);                                // print it out the serial monitor
            header += c;
            if (c == '\n') {                                // if the byte is a newline character
                // if the current line is blank, you got two newline characters in a row.
                // that's the end of the client HTTP request, so send a response:
                if (currentLine.length() == 0) {
                    // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
                    // and a content-type so the client knows what's coming, then a blank line:
                    client.println("HTTP/1.1 200 OK");
                    client.println("Content-type:text/html");
                    client.println("Connection: close");
                    client.println();

                    // Display the HTML web page
                    client.println("<!DOCTYPE html><html>");
                    client.println("<head>");
                    client.println("<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
                    client.println("<meta http-equiv=\"refresh\" content=15>");
                    client.println("<body>");
                    client.println("<table border=0>");
                    client.println("  <tr>");
                    client.println("    <td colspan=2 align=center><font size=6>ESP8266<br>Temperature Sensor<br>Monitor</font></td>");
                    client.println("  </tr>");
                    client.println("  <tr>");
                    client.println("  <td colspan=2><hr></td>");
                    client.println("  </tr>");
                    client.println("  <tr>");
                    client.println("    <th align=center><font size=5>Sensor</font></th>");
                    client.println("    <th align=center><font size=5>Temperature</font></th>");
                    client.println("  </tr>");
                    for(i = 0; i < numberOfDevices; i = i + 1) {
                        // Search the wire for address
                        client.println("  <tr>");
                        if(sensors.getAddress(tempDeviceAddress, i)){
                            client.println("    <td align=center>");
                            client.print(addr2s(tempDeviceAddress, s, sizeof(s)));
                            client.print("</td>");
                            client.println("    <td align=center>");
                            tempC = sensors.getTempC(tempDeviceAddress);
                            client.print(DallasTemperature::toFahrenheit(tempC));
                            client.print("</td>");
                            }
                        client.println("  </tr>");
                        } // for
                    client.println("</table>");
                    client.println("</body>");
                    client.println("</html>");

                    // The HTTP response ends with another blank line
                    client.println();
                    // Break out of the while loop
                    break;
                    }
                else { // if you got a newline, then clear currentLine
                    currentLine = "";
                    }
                }
            else if (c != '\r') {  // if you got anything else but a carriage return character,
                currentLine += c;      // add it to the end of the currentLine
                }
            }
        }
    // Clear the header variable
    header = "";
    // Close the connection
    client.stop();
    Serial.println("Client disconnected.");
    Serial.println("");
    }

}
